/*
 * (c) 2005 by Simon Gander <simongander@gmx.ch>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package guettingen;

class Block extends SpurElement {
	public Block () {
		
	}
	
	public void setzeParameter (Lampe wv, Lampe rv, Lampe wn, Lampe rn, Lampe rm, Lampe hfb, Lampe af) {
		weiss_von = wv;
		rot_von = rv;
		weiss_nach = wn;
		rot_nach = rn;
		rueckmelden = rm;
		hat_freie_bahn = hfb;
		freie_bahn_angefordert = af;
		setzeStatus(0);
	}
	
	public void freieBahnAnfordern () {
		if (status != 0) return;
		habe_freie_bahn = true;
		setzeStatus (0);
	}

	public void freieBahnAngefordert () {
		if (status != 0) return;
		if (fb_festgehalten) {
			freie_bahn_angefordert.einschalten();
			angefordert = true;
			return;
		}
		freie_bahn_angefordert.ausschalten();
		habe_freie_bahn = false;
		angefordert = false;
		setzeStatus (0);
	}

	public void freieBahnFesthalten () {
		if (!habe_freie_bahn) return;
		fb_festgehalten = !fb_festgehalten;
		if (angefordert && !fb_festgehalten) {
			freieBahnAngefordert();
		}
		// An dieser Stelle k?nnte man nat?rlich noch die Taste updaten...
	}
	
	public boolean vorblocken () {
		if (!habe_freie_bahn) return false;
		setzeStatus(1);
		return true;
	}

	public boolean vorgeblockt () {
		if (habe_freie_bahn) return false;
		setzeStatus (1);
		return true;
	}
	
	public boolean blocken () {
		if (!habe_freie_bahn) return false;
		if (status != 1) return false;
		setzeStatus(2);
		return true;
	}

	public boolean geblockt () {
		if (habe_freie_bahn) return false;
		if (status != 1) return false;
		setzeStatus(2);
		return true;
	}
	
	public void rueckmeldenMoeglich () {
		if (status != 2) return;
		if (habe_freie_bahn) return;
		rueck_moeglich = true;
		if (zug_eingefahren) {
			rueckmelden.einschalten();
		}
	}

	public void rueckmelden () {
		if (status != 2) return;
		if (habe_freie_bahn) return;
		if (!rueckmelden.ein) return;
		rueckmelden.ausschalten();
		rueck_moeglich = false;
		zug_eingefahren = false;
		setzeStatus(0);
	}
	
	public void rueckgemeldet () {
		if (status != 2) return;
		if (!habe_freie_bahn) return;
		setzeStatus(0);
		if (angefordert && !fb_festgehalten) {
			freieBahnAngefordert();
		}
	}
	
	public void setzeStatus (int s) {
		status = s;
		if (!habe_freie_bahn) {
			hat_freie_bahn.einschalten();
			weiss_nach.ausschalten();
			rot_nach.ausschalten();
			switch (status) {
				case 0:
				weiss_von.einschalten();
				rot_von.ausschalten();
				break;
				case 1:
				weiss_von.einschalten();
				rot_von.einschalten();
				break;
				case 2:
				weiss_von.ausschalten();
				rot_von.einschalten();
				break;
			}
		} else {
			hat_freie_bahn.ausschalten();
			weiss_von.ausschalten();
			rot_von.ausschalten();
			switch (status) {
				case 0:
				weiss_nach.einschalten();
				rot_nach.ausschalten();
				break;
				case 1:
				weiss_nach.einschalten();
				rot_nach.einschalten();
				break;
				case 2:
				weiss_nach.ausschalten();
				rot_nach.einschalten();
				break;
			}
		} 
	}
	
	public void freigeben (boolean richtung) {
		super.freigeben(richtung);
		
		if (!richtung && (links == null)) {
			rueckgemeldet();
		}
		
		if (richtung && (rechts == null)) {
			rueckgemeldet();
		}
	}
	
	public int signalStellung(boolean richtung) {
		if (richtung) {
			if (rechts == null) return 1;
			return rechts.signalStellung(true);
		} else {
			if (links == null) return 1;
			return links.signalStellung(false);
		}
	}

	public void zugEingefahren() {
		zug_eingefahren = true;
		if (rueck_moeglich) {
			rueckmelden.einschalten();
		}
	}
	
	Lampe weiss_von;
	Lampe rot_von;
	Lampe weiss_nach;
	Lampe rot_nach;
	Lampe rueckmelden;
	Lampe hat_freie_bahn;
	Lampe freie_bahn_angefordert;
	
	boolean habe_freie_bahn;
	boolean fb_festgehalten;

	boolean rueck_moeglich;
	boolean zug_eingefahren;

	int status;	// 0 = Frei, 1 = Vorgeblockt, 2 = Geblockt
	boolean angefordert;
}
 
